const path = require('path');
const readline = require('readline')
const chokidar = require('chokidar');
const execa = require('execa')
const {
  done,
  info,
  error
} = require('@vue/cli-shared-utils')
const bundleRun = require('./lib/bundleRun');
module.exports = (api, options) => {
  const expressOptions = (options.pluginOptions && options.pluginOptions.express) || {};
  const entryDev= expressOptions.entryDev || './src/server';
  const watchDir= expressOptions.watchDir || './src/**';
  api.registerCommand(
    'express:serve',
    {
      description: 'serve vue app and express on live',
      usage: 'vue-cli-service express:serve',
      details: 'Will bundle your vue app and express code that use es5/6/7 syntax and inject to vue webpack-dev-server'
    },
    async (args, rawArgs) => {
      const basePath=path.resolve(api.resolve('./'),'vm.js');
      let serve = null;
      async function start() {
        return await bundleRun(api,expressOptions.entryDev,basePath).then(srv => {
          return api.configureDevServer((app, server) => {
            srv(app,server.listeningApp);
            info('Express code inject success!');
          });
        }).then(()=>{
          return api.service.run('serve', {// Run the serve command
            _: [],
            // Use dashboard if called from ui
            dashboard: args.dashboard,
            https: args.https
          });
        }).then(res=>{
          serve=res.server;
        }).catch(err=>{
          error(err);
          process.exit(1);
        })
      }

      function stop() {
        if (serve) serve.close();
        serve = null;
      }

      // Handle Ctrl+C on Windows
      if (process.platform === 'win32') {
        readline
          .createInterface({
            input: process.stdin,
            output: process.stdout
          })
          .on('SIGINT', () => {
            process.exit(0)
          })
      }

      return start().then(()=>{
        // watch file changes
        chokidar
          .watch(path.resolve(api.resolve('./'),watchDir))
          .on('change', (file,stats) => {
            if (serve) {
              info(`Detect ${file} changed, restarting Development Server...`);
              stop();
              setTimeout(start, 1000);
            } else start();
          });
      });
    }
  );
};

module.exports.defaultModes = {
  'express:build': 'production',
  'express:serve': 'development'
}